==> mongodb-6.0.25.gh.tar.gz: mongo-r6.0.25/jstests/aggregation/bugs/server6195.js <==
// ensure $concat asserts on string

load('jstests/aggregation/extras/utils.js');
load('jstests/libs/sbe_assert_error_override.js');  // Override error-code-checking APIs.

c = db.s6570;
c.drop();
c.save({v: "$", w: ".", x: "foo", y: "bar", z: "z\0z"});

assert.eq(c.aggregate({$project: {str: {$concat: ["X", "$x", "Y", "$y"]}}}).toArray()[0].str,
          "XfooYbar");
assert.eq(c.aggregate({$project: {str: {$concat: ["$v", "X", "$w", "Y"]}}}).toArray()[0].str,
          "$X.Y");
assert.eq(c.aggregate({$project: {str: {$concat: ["$w", "X", "$v", "Y"]}}}).toArray()[0].str,
          ".X$Y");
assert.eq(c.aggregate({$project: {str: {$concat: ["X", "$z", "a\0a", "Y"]}}}).toArray()[0].str,
          "Xz\0za\0aY");

// Nullish (both with and without other strings)
assert.isnull(c.aggregate({$project: {str: {$concat: ["$missing"]}}}).toArray()[0].str);
assert.isnull(c.aggregate({$project: {str: {$concat: [null]}}}).toArray()[0].str);
assert.isnull(c.aggregate({$project: {str: {$concat: [undefined]}}}).toArray()[0].str);
assert.isnull(c.aggregate({$project: {str: {$concat: ["$x", "$missing", "$y"]}}}).toArray()[0].str);
assert.isnull(c.aggregate({$project: {str: {$concat: ["$x", null, "$y"]}}}).toArray()[0].str);
assert.isnull(c.aggregate({$project: {str: {$concat: ["$x", undefined, "$y"]}}}).toArray()[0].str);

// assert fail for all other types
assertErrorCode(c, {$project: {str: {$concat: [MinKey]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [1]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [NumberInt(1)]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [NumberLong(1)]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [true]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [function() {}]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [{}]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [[]]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [new Timestamp(0, 0)]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [new Date(0)]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [new BinData(0, "")]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [/asdf/]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [MaxKey]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [new ObjectId()]}}}, 16702);

==> mongodb-8.0.12.gh.tar.gz: mongo-r8.0.12/jstests/aggregation/bugs/server6195.js <==
// ensure $concat asserts on string

import "jstests/libs/sbe_assert_error_override.js";

import {assertErrorCode} from "jstests/aggregation/extras/utils.js";

let c = db.s6570;
c.drop();
c.save({v: "$", w: ".", x: "foo", y: "bar", z: "z\0z"});

assert.eq(c.aggregate({$project: {str: {$concat: ["X", "$x", "Y", "$y"]}}}).toArray()[0].str,
          "XfooYbar");
assert.eq(c.aggregate({$project: {str: {$concat: ["$v", "X", "$w", "Y"]}}}).toArray()[0].str,
          "$X.Y");
assert.eq(c.aggregate({$project: {str: {$concat: ["$w", "X", "$v", "Y"]}}}).toArray()[0].str,
          ".X$Y");
assert.eq(c.aggregate({$project: {str: {$concat: ["X", "$z", "a\0a", "Y"]}}}).toArray()[0].str,
          "Xz\0za\0aY");

// Nullish (both with and without other strings)
assert.isnull(c.aggregate({$project: {str: {$concat: ["$missing"]}}}).toArray()[0].str);
assert.isnull(c.aggregate({$project: {str: {$concat: [null]}}}).toArray()[0].str);
assert.isnull(c.aggregate({$project: {str: {$concat: [undefined]}}}).toArray()[0].str);
assert.isnull(c.aggregate({$project: {str: {$concat: ["$x", "$missing", "$y"]}}}).toArray()[0].str);
assert.isnull(c.aggregate({$project: {str: {$concat: ["$x", null, "$y"]}}}).toArray()[0].str);
assert.isnull(c.aggregate({$project: {str: {$concat: ["$x", undefined, "$y"]}}}).toArray()[0].str);

// assert fail for all other types
assertErrorCode(c, {$project: {str: {$concat: [MinKey]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [1]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [NumberInt(1)]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [NumberLong(1)]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [true]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [function() {}]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [{}]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [[]]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [new Timestamp(0, 0)]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [new Date(0)]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [new BinData(0, "")]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [/asdf/]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [MaxKey]}}}, 16702);
assertErrorCode(c, {$project: {str: {$concat: [new ObjectId()]}}}, 16702);
